part of jaguar_auth.authenticators;

/// An [Authenticator] for standard username password form style login.
/// It expects a `application/x-www-form-urlencoded` encoded body where the
/// username and password form fields must be called `username` and `password`
/// respectively.
///
/// Arguments:
/// It uses [sessionManager] to read session information from the request
/// and update session information to response
///
/// It uses [modelManager] to fetch user model for the authentication request
/// and authenticate against the password
///
/// Outputs ans Variables:
/// The authenticated user model is injected into the context as input
///
/// [sessionManager] is added to context variable for usage in route handler or
/// other interceptors
class FormAuth extends Interceptor {
  /// Session manager is used to read session information from the request
  /// and update session information to response
  final SessionManager sessionManager;

  /// Model manager is used to fetch user model for the authentication request
  /// and authenticate against the password
  final AuthModelManager modelManager;

  /// The key by which authorizationId shall be stored in session data
  final String authorizationIdKey;

  /// Specifies whether the interceptor should create/update the session data
  /// on authentication success.
  ///
  /// If set to false, session creation and update must be done manually
  final bool manageSession;

  FormAuth(this.modelManager,
      {this.authorizationIdKey: 'id',
      this.manageSession: true,
      SessionManager sessionManager})
      : sessionManager =
            sessionManager != null ? sessionManager : new CookieSession();

  /// Parses the session from request, fetches the user model and authenticates
  /// it against the password.
  ///
  /// On successful login, injects authenticated user model as context input and
  /// session manager as context variable.
  Future<AuthorizationUser> pre(Context ctx) async {
    Map<String, String> form = await ctx.req.bodyAsUrlEncodedForm();

    if (form is! Map<String, String>) {
      throw new UnAuthorizedError();
    }

    final String username = form['username'];
    final String password = form['password'];

    if (username is! String) {
      throw new UnAuthorizedError();
    }

    if (password is! String) {
      throw new UnAuthorizedError();
    }

    final subject = await modelManager.authenticate(ctx, username, password);

    if (subject == null) {
      throw new UnAuthorizedError();
    }

    if (manageSession is bool && manageSession) {
      final data = <String, String>{
        authorizationIdKey: subject.authorizationId
      };
      await sessionManager.createSession(data);
    }

    ctx.addVariable(SessionManager);

    return subject;
  }

  /// Writes session information into the response
  Future<Response> post(Context ctx, Response resp) async {
    return await sessionManager.writeResponse(ctx.req, resp);
  }
}
